/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2007,2008 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: DistributedObjectInfo.java 2438 2010-02-24 16:20:30Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.server;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import net.jcip.annotations.ThreadSafe;

import org.ow2.cmi.controller.common.ImmutableClusteredObjectInfos;
import org.ow2.cmi.controller.server.filter.IFilter;
import org.ow2.cmi.lb.data.PolicyData;
import org.ow2.util.pool.api.IPoolConfiguration;

/**
 * Represents the replicated data for a clustered object.
 * @author The new CMI team
 */
@ThreadSafe
public final class DistributedObjectInfo extends ImmutableClusteredObjectInfos
implements Serializable, Cloneable {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = 7128398689206804315L;

    /**
     * Data of the policy.
     */
    private volatile PolicyData policyData;

    /**
     * Filters to restrict access on this object.
     */
    private final List<IFilter> filters =
        Collections.synchronizedList(new ArrayList<IFilter>());

    /**
     * True if this object has a state replicated.
     */
    private final boolean replicated;

    /**
     * Configuration of the pool.
     */
    private volatile IPoolConfiguration poolConfiguration;

    /**
     * Constructs informations for a clustered object with the default strategy for the given LB policy.
     * @param clusterName a name of the interface that the object implements
     * @param proxyEnabled true if a CMI proxy should be used to make invocations
     * @param objectName a name of object
     * @param itfName a name of the interface that the object implements
     * @param businessName a name of the business interface that the object implements
     * @param policyData data of the LB policy
     * @param stateful true if this object has a state
     * @param replicated true if this object has a state replicated
     * @param applicationExceptionNames classnames of the application exceptions
     * @param poolConfiguration configuration of the pool
     */
    public DistributedObjectInfo(
            final String clusterName,
            final boolean proxyEnabled,
            final String objectName,
            final String itfName,
            final String businessName,
            final PolicyData policyData,
            final boolean stateful,
            final boolean replicated,
            final Set<String> applicationExceptionNames,
            final IPoolConfiguration poolConfiguration) {
        super(clusterName, proxyEnabled, objectName, itfName,
                businessName, stateful, applicationExceptionNames);
        this.policyData = policyData;
        this.replicated = replicated;
        this.poolConfiguration = poolConfiguration;
    }

    /**
     * @param policyData data of the associated policy
     */
    public void setPolicyData(final PolicyData policyData) {
        this.policyData = policyData;
    }

    /**
     * @return data of the associated policy
     */
    public PolicyData getPolicyData() {
        return policyData;
    }

    /**
     * @return true if this object has a state replicated
     */
    public boolean isReplicated() {
        return replicated;
    }

    /**
     * @return configuration of the pool
     */
    public IPoolConfiguration getPoolConfiguration() {
        return poolConfiguration;
    }

    /**
     * Set the configuration for the pool of CMIReferenceable.
     * @param poolConfiguration the configuration for the pool of CMIReferenceable
     */
    public void setPoolConfiguration(final IPoolConfiguration poolConfiguration) {
        this.poolConfiguration = poolConfiguration;
    }

    /**
     * @return a list of filters
     */
    public List<? extends IFilter> getFilters() {
        return filters;
    }

    /**
     * Add a new filter.
     * @param filter a filter to add
     */
    public void addFilter(final IFilter filter) {
        filters.add(filter);
    }

    /**
     * Remove a filter.
     * @param filter a filter to remove
     * @return true if the filter has been removed
     */
    public boolean removeFilter(final IFilter filter) {
        return filters.remove(filter);
    }

    @Override
    public boolean equals(final Object object) {
        if(object == null || !(object instanceof DistributedObjectInfo)) {
            return false;
        }
        DistributedObjectInfo distributedObjectInfo = (DistributedObjectInfo) object;
        return super.equals(distributedObjectInfo)
            && (policyData == null ?
                    distributedObjectInfo.policyData == null :
                        policyData.equals(distributedObjectInfo.policyData))
            && replicated == distributedObjectInfo.replicated
            && (poolConfiguration == null ?
                    distributedObjectInfo.poolConfiguration == null :
                        poolConfiguration.equals(distributedObjectInfo.poolConfiguration))
            && filters.equals(distributedObjectInfo.filters);
    }

    @Override
    public int hashCode() {
        return super.hashCode()
            + (poolConfiguration == null ? 0 : poolConfiguration.hashCode())
            + (policyData == null ? 0 : policyData.hashCode())
            + Boolean.valueOf(replicated).hashCode()
            + filters.hashCode();
    }

    @Override
    public String toString() {
        return "DistributedObjectInfo=["
             + super.toString()
            + ",lbPolicyData=" + policyData
            + ",filters= " + filters
            + ",replicated=" + replicated
            + ",poolConfiguration=" + poolConfiguration
            + "]";
    }

    @Override
    public DistributedObjectInfo clone() throws CloneNotSupportedException {
        DistributedObjectInfo distributedObjectInfo =
            (DistributedObjectInfo) super.clone();
        distributedObjectInfo.policyData = policyData.clone();
        return distributedObjectInfo;
    }

    public ImmutableClusteredObjectInfos getImmutableInfos() {
        return new ImmutableClusteredObjectInfos(getClusterName(), isProxyEnabled(),
                getObjectName(), getItfName(), getBusinessName(), isStateful(), getApplicationExceptionNames());
    }

}
