/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008-2009 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: ServerView.java 2438 2010-02-24 16:20:30Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.cmi.controller.server;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import net.jcip.annotations.NotThreadSafe;

import org.ow2.cmi.reference.ServerRef;

/**
 * Wraps the informations related to a server.
 * @author Loris Bouzonnet
 */
@NotThreadSafe
public class ServerView implements Serializable {

    /**
     * Id for serializable class.
     */
    private static final long serialVersionUID = -2744394720016534529L;

    /**
     * Default load factor.
     */
    private static final int DEFAULT_LOAD_FACTOR = 100;

    /**
     * Server reference.
     */
    private final ServerRef serverRef;

    /**
     * Indicates if this server is blacklisted.
     */
    private boolean blacklisted = false;

    /**
     * Load-factor.
     */
    private int loadFactor = DEFAULT_LOAD_FACTOR;

    /**
     * Server id.
     */
    private final short id;

    /**
     * Server name.
     */
    private final String serverName;

    /**
     * Domains whose this server belongs.
     */
    private Map<String, Domain> domains = new HashMap<String, Domain>();


    /**
     * Wrap the informations related to a server.
     * @param serverRef Server reference.
     * @param id Server id.
     * @param serverName Server name.
     */
    public ServerView(final ServerRef serverRef, final short id, final String serverName) {
        this.serverRef = serverRef;
        this.id = id;
        this.serverName = serverName;
    }

    /**
     * @return Server reference.
     */
    public ServerRef getServerRef() {
        return serverRef;
    }

    /**
     * Set the load-factor.
     * @param loadFactor the load-factor.
     */
    public void setLoadFactor(final int loadFactor) {
        this.loadFactor = loadFactor;
    }

    /**
     * @return true if this server is blacklisted.
     */
    public boolean isBlacklisted() {
        return blacklisted;
    }

    /**
     * Set if this server is blacklisted.
     * @param blacklisted true if this server is blacklisted.
     */
    public void setBlacklisted(final boolean blacklisted) {
        this.blacklisted = blacklisted;
    }

    /**
     * @return Load-factor.
     */
    public int getLoadFactor() {
        return loadFactor;
    }

    /**
     * @return Server id.
     */
    public short getId() {
        return id;
    }

    /**
     * @return Server name.
     */
    public String getServerName() {
        return serverName;
    }

    /**
     * Returns the informations related to this server in the given domain.
     * @param domainName a domain name
     * @return the informations related to this server in the given domain.
     */
    public Domain getDomain(final String domainName) {
        return domains.get(domainName);
    }

    /**
     * Add a domain to which this server belongs.
     * @param domain a domain
     */
    public void addDomain(final Domain domain) {
        domains.put(domain.getName(), domain);
    }

    @Override
    public String toString() {
        return "ServerView[blacklisted=" + blacklisted
            + ", loadFactor=" + loadFactor + ", id=" + id
            + ", serverName=" + serverName + ", domains=" + domains + "]";
    }

}
